@@ -13,13 +13,17 @@ the full text of the license.
 
 class PackageInfo:
     def get_version(self, package):
-        '''Return the installed version of a package.'''
-
+        '''Return the installed version of a package.
+        
+        Throw ValueError if package does not exist.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_available_version(self, package):
-        '''Return the latest available version of a package.'''
-
+        '''Return the latest available version of a package.
+        
+        Throw ValueError if package does not exist.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_dependencies(self, package):
@@ -28,27 +32,35 @@ class PackageInfo:
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_source(self, package):
-        '''Return the source package name for a package.'''
-
+        '''Return the source package name for a package.
+        
+        Throw ValueError if package does not exist.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def is_distro_package(self, package):
-        '''Check if a package is a genuine distro package (True) or comes from
-        a third-party source.'''
+        '''Check package origin.
 
+        Return True if the package is a genuine distro package, or False if it
+        comes from a third-party source.
+
+        Throw ValueError if package does not exist.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_architecture(self, package):
         '''Return the architecture of a package.
 
         This might differ on multiarch architectures (e. g.  an i386 Firefox
-        package on a x86_64 system)'''
-
+        package on a x86_64 system)
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_files(self, package):
-        '''Return list of files shipped by a package.'''
+        '''Return list of files shipped by a package.
 
+        Throw ValueError if package does not exist.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_modified_files(self, package):
@@ -57,35 +69,40 @@ class PackageInfo:
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_file_package(self, file, uninstalled=False, map_cachedir=None):
-        '''Return the package a file belongs to, or None if the file is not
-        shipped by any package.
+        '''Return the package a file belongs to.
+        
+        Return None if the file is not shipped by any package.
         
         If uninstalled is True, this will also find files of uninstalled
         packages; this is very expensive, though, and needs network access and
         lots of CPU and I/O resources. In this case, map_cachedir can be set to
         an existing directory which will be used to permanently store the
-        downloaded maps. If it is not set, a temporary directory will be used.'''
-
+        downloaded maps. If it is not set, a temporary directory will be used.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_system_architecture(self):
-        '''Return the architecture of the system, in the notation used by the
-        particular distribution.'''
-
+        '''Return the architecture of the system.
+        
+        This should use the notation of the particular distribution.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def set_mirror(self, url):
-        '''Explicitly set a distribution mirror URL for operations that need to
-        fetch distribution files/packages from the network.
+        '''Explicitly set a distribution mirror URL.
+        
+        This might be called for operations that need to fetch distribution
+        files/packages from the network.
 
         By default, the mirror will be read from the system configuration
-        files.'''
-
+        files.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def get_source_tree(self, srcpackage, dir, version=None):
-        '''Download given source package and unpack it into dir (which should
-        be empty).
+        '''Download a source package and unpack it into dir..
+        
+        dir should exist and be empty.
 
         This also has to care about applying patches etc., so that dir will
         eventually contain the actually compiled source.
@@ -95,15 +112,15 @@ class PackageInfo:
 
         Return the directory that contains the actual source root directory
         (which might be a subdirectory of dir). Return None if the source is
-        not available.'''
-
+        not available.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def compare_versions(self, ver1, ver2):
         '''Compare two package versions.
 
-        Return -1 for ver < ver2, 0 for ver1 == ver2, and 1 for ver1 > ver2.'''
-
+        Return -1 for ver < ver2, 0 for ver1 == ver2, and 1 for ver1 > ver2.
+        '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
     def enabled(self):
@@ -152,4 +169,9 @@ class PackageInfo:
         '''
         raise NotImplementedError, 'this method must be implemented by a concrete subclass'
 
+    def package_name_glob(self, glob):
+        '''Return known package names which match given glob.'''
+
+        raise NotImplementedError, 'this method must be implemented by a concrete subclass'
+
 import packaging_impl
