@@ -11,7 +11,7 @@ option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 the full text of the license.
 '''
 
-import subprocess, os, glob, stat, sys, tempfile, glob, re
+import subprocess, os, glob, stat, sys, tempfile, glob, re, shutil
 
 import warnings
 warnings.filterwarnings('ignore', 'apt API not stable yet', FutureWarning)
@@ -27,6 +27,8 @@ class __AptDpkgPackageInfo(PackageInfo):
         self._apt_cache = None
         self._contents_dir = None
         self._mirror = None
+        # Checking whether we should use the API from python-apt 0.7.9
+        self.apt_pre_079 = not hasattr(apt.Package, 'installed')
 
         self.configuration = '/etc/default/apport'
 
@@ -38,32 +40,51 @@ class __AptDpkgPackageInfo(PackageInfo):
         except AttributeError:
             pass
 
-    def _cache(self, package):
-        '''Return apt.Cache()[package] (initialized lazily).
+    def _cache(self):
+        '''Return apt.Cache() (initialized lazily).
         
-        Throw a ValueError if the package does not exist.'''
-
+        Throw a ValueError if the package does not exist.
+        '''
         if not self._apt_cache:
             self._apt_cache = apt.Cache()
+        return self._apt_cache
+
+    def _apt_pkg(self, package):
+        '''Return apt.Cache()[package] (initialized lazily).
+        
+        Throw a ValueError if the package does not exist.
+        '''
         try:
-            return self._apt_cache[package]
+            return self._cache()[package]
         except KeyError:
             raise ValueError, 'package does not exist'
 
     def get_version(self, package):
         '''Return the installed version of a package.'''
 
-        return self._cache(package).installedVersion
+        pkg = self._apt_pkg(package)
+        if self.apt_pre_079:
+            if not pkg.isInstalled:
+                raise ValueError, 'package does not exist'
+            return pkg.installedVersion
+        else:
+            inst = pkg.installed
+            if not inst:
+                raise ValueError, 'package does not exist'
+            return inst.version
 
     def get_available_version(self, package):
         '''Return the latest available version of a package.'''
 
-        return self._cache(package).candidateVersion
+        if self.apt_pre_079:
+            return self._apt_pkg(package).candidateVersion
+        else:
+            return self._apt_pkg(package).candidate.version
 
     def get_dependencies(self, package):
         '''Return a list of packages a package depends on.'''
 
-        cur_ver = self._cache(package)._pkg.CurrentVer
+        cur_ver = self._apt_pkg(package)._pkg.CurrentVer
         if not cur_ver:
             # happens with virtual packages
             return []
@@ -73,7 +94,15 @@ class __AptDpkgPackageInfo(PackageInfo):
     def get_source(self, package):
         '''Return the source package name for a package.'''
 
-        return self._cache(package).sourcePackageName
+        if self.apt_pre_079:
+            return self._apt_pkg(package).sourcePackageName
+        else:
+            if self._apt_pkg(package).installed:
+                return self._apt_pkg(package).installed.source_name
+            elif self._apt_pkg(package).candidate:
+                return self._apt_pkg(package).candidate.source_name
+            else:
+                raise ValueError, 'package %s does not exist' % package
 
     def is_distro_package(self, package):
         '''Check if a package is a genuine distro package (True) or comes from
@@ -84,14 +113,33 @@ class __AptDpkgPackageInfo(PackageInfo):
         this_os = lsb_release.communicate()[0].strip()
         assert lsb_release.returncode == 0
 
-        if self._cache(package).installedVersion is None:
-            return False # LP#252734
+        pkg = self._apt_pkg(package)
+        # some PPA packages have installed version None, see LP#252734
+        if self.apt_pre_079:
+            if pkg.isInstalled and pkg.installedVersion is None:
+                return False
+        else:
+            if pkg.installed and pkg.installed.version is None:
+                return False
 
-        origins = self._cache(package).candidateOrigin
+        native_origins = [this_os]
+        try:
+            for f in os.listdir('/etc/apport/native-origins.d'):
+                for line in open(f):
+                    line = line.strip()
+                    if line:
+                        native_origins.append(line)
+        except OSError:
+            pass
+
+        origins = None
+        if self.apt_pre_079:
+            origins = pkg.candidateOrigin
+        else:
+            origins = pkg.candidate.origins
         if origins: # might be None
             for o in origins:
-                # note: checking site for ppa is a hack until LP #140412 gets fixed
-                if o.origin == this_os and not o.site.startswith('ppa'):
+                if o.origin in native_origins:
                     return True
         return False
 
@@ -101,7 +149,15 @@ class __AptDpkgPackageInfo(PackageInfo):
         This might differ on multiarch architectures (e. g.  an i386 Firefox
         package on a x86_64 system)'''
 
-        return self._cache(package).architecture or 'unknown'
+        if self.apt_pre_079:
+            return self._apt_pkg(package).architecture or 'unknown'
+        else:
+            if self._apt_pkg(package).installed:
+                return self._apt_pkg(package).installed.architecture or 'unknown'
+            elif self._apt_pkg(package).candidate:
+                return self._apt_pkg(package).candidate.architecture or 'unknown'
+            else:
+                raise ValueError, 'package %s does not exist' % package
 
     def get_files(self, package):
         '''Return list of files shipped by a package.'''
@@ -122,7 +178,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                 raise OSError
             max_time = max(s.st_mtime, s.st_ctime)
         except OSError:
-            return [listfile]
+            return []
 
         # create a list of files with a newer timestamp for md5sum'ing
         sums = ''
@@ -138,7 +194,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                     print >> sys.stderr, 'WARNING:', sumfile, 'contains NUL character, ignoring line'
                     continue
                 words  = line.split()
-                if len(line) < 1:
+                if not words:
                     print >> sys.stderr, 'WARNING:', sumfile, 'contains empty line, ignoring line'
                     continue
                 s = os.stat('/' + words[-1])
@@ -286,6 +342,44 @@ class __AptDpkgPackageInfo(PackageInfo):
         # TODO: Ubuntu specific
         return 'linux-image-' + os.uname()[2]
 
+    def _install_debug_kernel(self, report):
+        '''Install kernel debug package 
+
+        Ideally this would be just another package but the kernel is
+        special in various ways currently so we can not use the apt
+        method.
+        '''
+        import urllib, apt_pkg
+        installed = []
+        outdated = []
+        kver = report['Uname'].split()[1]
+        arch = report['Architecture']
+        ver = report['Package'].split()[1]
+        debug_pkgname = 'linux-image-debug-%s' % kver
+        c = self._cache()
+        if c.has_key(debug_pkgname) and c[debug_pkgname].isInstalled:
+            #print 'kernel ddeb already installed'
+            return (installed, outdated)
+        target_dir = apt_pkg.Config.FindDir('Dir::Cache::archives')+'/partial'
+        deb = '%s_%s_%s.ddeb' % (debug_pkgname, ver, arch)
+        # FIXME: this package is currently not in Packages.gz
+        url = 'http://ddebs.ubuntu.com/pool/main/l/linux/%s' % deb
+        out = open(os.path.join(target_dir, deb), 'w')
+        # urlretrieve does not return 404 in the headers so we use urlopen
+        u = urllib.urlopen(url)
+        if u.getcode() > 400:
+            raise IOError, 'urllib returned %s for %s' % (u.getcode(), url)
+        while True:
+            block = u.read(8*1024)
+            if not block:
+                break
+            out.write(block)
+        out.flush()
+        ret = subprocess.call(['dpkg', '-i', os.path.join(target_dir, deb)])
+        if ret == 0:
+            installed.append(deb.split('_')[0])
+        return (installed, outdated)
+
     def install_retracing_packages(self, report, verbosity=0,
             unpack_only=False, no_pkg=False, extra_packages=[]):
         '''Install packages which are required to retrace a report.
@@ -302,8 +396,11 @@ class __AptDpkgPackageInfo(PackageInfo):
         
         Return a tuple (list of installed packages, string with outdated packages).
         '''
-        self._cache('bash') # ensure that cache is initialized
-        c = self._apt_cache
+        if (report['ProblemType'] == 'KernelCrash' and 
+            report['Package'].startswith('linux-image')):
+            return self._install_debug_kernel(report)
+
+        c = self._cache()
 
         try:
             if verbosity:
@@ -336,13 +433,9 @@ class __AptDpkgPackageInfo(PackageInfo):
             (pkg, version) = l.split()[:2]
             dependency_versions[pkg] = version
             try:
-                # this fails for packages which are still installed, but gone from
-                # the archive; i. e. /var/lib/dpkg/status still knows about them
-                if not c[pkg]._lookupRecord():
-                    raise KeyError
-                if not 'Architecture: all' in c[pkg]._records.Record:
+                if self.get_architecture(pkg) != 'all':
                     dependency_versions[pkg+'-dbgsym'] = dependency_versions[pkg]
-            except KeyError:
+            except ValueError:
                 print >> sys.stderr, 'WARNING: package %s not known to package cache' % pkg
 
         for pkg, ver in dependency_versions.iteritems():
@@ -351,16 +444,26 @@ class __AptDpkgPackageInfo(PackageInfo):
                 continue
 
             # ignore packages which are already installed in the right version
-            if (ver and c[pkg].installedVersion == ver) or \
-               (not ver and c[pkg].installedVersion):
-               continue
+            if self.apt_pre_079:
+                if (ver and c[pkg].isInstalled and c[pkg].installedVersion ==\
+                    ver) or (not ver and c[pkg].isInstalled):
+                    continue
+            else:
+                if (ver and c[pkg].installed and c[pkg].installed.version ==\
+                    ver) or (not ver and c[pkg].installed):
+                    continue
 
-            if ver and c[pkg].candidateVersion != ver:
+            candidate_version = None
+            if self.apt_pre_079:
+                candidate_version = c[pkg].candidateVersion
+            else:
+                candidate_version = c[pkg].candidate.version
+            if ver and candidate_version != ver:
                 if not pkg.endswith('-dbgsym'):
                     outdated += '%s: installed version %s, latest version: %s\n' % (
-                        pkg, ver, c[pkg].candidateVersion)
+                        pkg, ver, candidate_version)
                 print >> sys.stderr, 'WARNING: %s version %s required, but %s is available' % (
-                    pkg, ver, c[pkg].candidateVersion)
+                    pkg, ver, candidate_version)
                 if not unpack_only:
                     uninstallable.append (c[pkg].name)
                     continue
@@ -398,6 +501,18 @@ class __AptDpkgPackageInfo(PackageInfo):
         except IOError, e:
             pass # we will complain to the user later
 
+        # package hooks might reassign Package:, check that we have the originally crashing binary
+        for path in ('InterpreterPath', 'ExecutablePath'):
+            if path in report and not os.path.exists(report[path]):
+                pkg = self.get_file_package(report[path], True)
+                if pkg:
+                    print 'Installing extra package', pkg, 'to get', path
+                    c[pkg].markInstall(False)
+                else:
+                    err = 'current version of package %s does not contain the program %s any more' % (report['Package'], path)
+                    outdated += err + '\n'
+                    print >> sys.stderr, 'WARNING:', err
+
         # check list of libraries that the crashed process referenced at
         # runtime and warn about those which are not available
         libs = set()
@@ -446,7 +561,12 @@ class __AptDpkgPackageInfo(PackageInfo):
             if os.geteuid() != 0:
                 print >> sys.stderr, 'You either need to call this program as root or install these packages manually:'
             for p in c.getChanges():
-                print >> sys.stderr, '  %s %s' % (p.name, p.candidateVersion)
+                if self.apt_pre_079:
+                    print >> sys.stderr, '  %s %s' % (p.name,
+                                                      p.candidateVersion)
+                else:
+                    print >> sys.stderr, '  %s %s' % (p.name,
+                                                      p.candidate.version)
 
         return (installed, outdated)
 
@@ -462,6 +582,10 @@ class __AptDpkgPackageInfo(PackageInfo):
             so = subprocess.PIPE
         subprocess.call(['dpkg', '-P'] + packages, stdout=so)
 
+    def package_name_glob(self, nameglob):
+        '''Return known package names which match given glob.'''
+
+        return glob.fnmatch.filter(self._cache().keys(), nameglob)
 
     #
     # Internal helper methods
@@ -695,7 +819,7 @@ impl = __AptDpkgPackageInfo()
 #
 
 if __name__ == '__main__':
-    import unittest, gzip, shutil
+    import unittest, gzip
 
     class _AptDpkgPackageInfoTest(unittest.TestCase):
 
@@ -741,6 +865,7 @@ if __name__ == '__main__':
 
             self.assert_(impl.get_version('libc6').startswith('2'))
             self.assertRaises(ValueError, impl.get_version, 'nonexisting')
+            self.assertRaises(ValueError, impl.get_version, 'wukrainian')
 
         def test_get_available_version(self):
             '''get_available_version().'''
@@ -776,7 +901,7 @@ if __name__ == '__main__':
 
             self.assertRaises(ValueError, impl.get_source, 'nonexisting')
             self.assertEqual(impl.get_source('bash'), 'bash')
-            self.assertEqual(impl.get_source('libc6'), 'glibc')
+            self.assert_('glibc' in impl.get_source('libc6'))
 
         def test_is_distro_package(self):
             '''is_distro_package().'''
@@ -921,6 +1046,14 @@ bo/gu/s                                                 na/mypackage
 
             self.assert_('linux' in impl.get_kernel_package())
 
+        def test_package_name_glob(self):
+            '''package_name_glob().'''
+
+            self.assert_(len(impl.package_name_glob('a*')) > 5)
+            self.assert_('bash' in impl.package_name_glob('ba*h'))
+            self.assertEqual(impl.package_name_glob('bash'), ['bash'])
+            self.assertEqual(impl.package_name_glob('xzywef*'), [])
+
     # only execute if dpkg is available
     try:
         if subprocess.call(['dpkg', '--help'], stdout=subprocess.PIPE,
